/*
 * STM32F3-Discovery demo
 * 2013, Kai O.
 */

#include <stm32f30x.h>
#include <system_stm32f30x.h>
#include <stm32f30x_tim.h>
#include <stm32f30x_gpio.h>
#include <stm32f30x_rcc.h>

#include "serial.h"
//#include "i2cmaster.h"
#include "stdlib.h"
#include "main.h"

// Incremented each 1ms
volatile uint32_t Ticks;

#define CLOCK_FREQUENCY    (72 * MHZ)
#define CLOCK_TICKS_PER_MS (CLOCK_FREQUENCY / 1000)
#define CLOCK_TICKS_PER_US (CLOCK_TICKS_PER_MS / 1000)

// DWT Current PC Sampler Cycle Count Register
// \sa 11.5.1. Summary and description of the DWT registers
// Cortex-M3 Technical Reference Manual
#define CYCCNTENA  (1 << 0)
#define DWT_CTRL   (*((volatile uint32_t *)0xE0001000))
#define DWT_CYCCNT (*((volatile uint32_t *)0xE0001004))

static void CORTEX_SysTickInit()
{
   CoreDebug->DEMCR |= CoreDebug_DEMCR_TRCENA_Msk;

   // Enable cycle counter
   DWT_CTRL |= CYCCNTENA;
   DWT_CYCCNT = 0;
}

void delay_ms(uint32_t delayMs)
{
   volatile uint32_t startTick = Ticks;
   // Ueberlauf sicher durch unsigned wrap
   while ((Ticks - startTick) < delayMs);
}

void delay_us(uint32_t delayUs)
{
   uint32_t delayTicks = delayUs * CLOCK_TICKS_PER_US;
   volatile uint32_t startTick = DWT_CYCCNT;
   // Ueberlauf sicher durch unsigned wrap
   while ((DWT_CYCCNT - startTick) < delayTicks);
}

void err_wait()
{
   for (;;)
   {
     UserLedToggle(1);
     delay_ms(200);
   }
}

static void UserLedSetup()
{
   /* GPIOE Periph clock enable */
   RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOE, ENABLE);

   GPIO_InitTypeDef GPIO_InitStructure;
   GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
   GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
   GPIO_Init(GPIOE, &GPIO_InitStructure);
}

int main(void)
{
   __disable_irq();

   /* Clock init */
   SystemInit();

   CORTEX_SysTickInit();

   /* SysTick auf 1ms */
   SysTick_Config(CLOCK_TICKS_PER_MS);

   __enable_irq();

   UserLedSetup();

   uartInit();

   for (int i = 0; i < 8; ++i)
   {
      UserLedOn(i+1);
      delay_ms(100);
      UserLedOff(i+1);
   }

   for (;;);
}

/*
 * Called each ms
 */
void SysTick_Handler(void)
{
   Ticks++;

   if (Ticks > 1000 && (Ticks % 500) == 0)
   {
      UserLedToggle(2);
   }
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  printf("Wrong parameters value: file %s on line %d\r\n", file, line);

  /* Infinite loop */
  for (;;)
  {
     /* Alarm: Flash all LED's */
     for (int i = 0; i < 8; ++i) UserLedToggle(i);
     delay_ms(100);
  }
}
#endif
